@@ -1,5 +1,7 @@
 import os
-import sys; sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
+import sys
+sys.path.insert(
+    0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
 import cv2
 import numpy as np
 import six
@@ -14,7 +16,7 @@ def load_npy_dict_file(fname):
 
     if os.path.exists(fname):
         npy_dict = np.load(fname, allow_pickle=True).item()
-        assert type(npy_dict) == dict
+        assert type(npy_dict) == dict or npy_dict is None
         return npy_dict
     else:
         return None
@@ -117,8 +119,6 @@ def read_image(img_path, target_size=256, crop_size=224):
             img = Image.open(f)
             img = img.convert('RGB')
             img = np.array(img)
-            # img = cv2.imread(img_path)
-
             img = resize_short(img, target_size, interpolation=None)
             img = crop_image(img, target_size=crop_size, center=True)
             # img = img[:, :, ::-1]
@@ -139,7 +139,10 @@ def _find_classes(dir):
     return classes, class_to_idx
 
 
-def get_typical_dataset_info(dataset_dir, subset="test", shuffle=False, random_seed=None):
+def get_typical_dataset_info(dataset_dir,
+                             subset="test",
+                             shuffle=False,
+                             random_seed=None):
     """
     where {dataset_dir}/{train,test,segmentations}}/{class1, class2, ...}/*.png exists.
 
@@ -153,7 +156,8 @@ def get_typical_dataset_info(dataset_dir, subset="test", shuffle=False, random_s
     Returns:
 
     """
-    IMG_EXTENSIONS = ('.jpg', '.jpeg', '.png', '.ppm', '.bmp', '.pgm', '.tif', '.tiff', '.webp')
+    IMG_EXTENSIONS = ('.jpg', '.jpeg', '.png', '.ppm', '.bmp', '.pgm', '.tif',
+                      '.tiff', '.webp')
 
     # read
     set_dir = os.path.join(dataset_dir, subset)
@@ -171,7 +175,10 @@ def get_typical_dataset_info(dataset_dir, subset="test", shuffle=False, random_s
                 continue
 
             image_paths.append(img_path)
-            seg_paths.append(os.path.join(seg_dir, img_path.split('test/')[-1].replace('jpg', 'png')))
+            seg_paths.append(
+                os.path.join(seg_dir,
+                             img_path.split('test/')[-1].replace('jpg',
+                                                                 'png')))
             labels.append(class_to_idx[class_name])
 
             assert os.path.exists(seg_paths[-1]), seg_paths[-1]
@@ -187,4 +194,58 @@ def get_typical_dataset_info(dataset_dir, subset="test", shuffle=False, random_s
         seg_paths = seg_paths[random_per]
         labels = labels[random_per]
 
-    return image_paths, seg_paths, labels, len(class_names)
\ No newline at end of file
+    return image_paths, seg_paths, labels, len(class_names)
+
+
+def restore_image(img):
+    mean = [0.485, 0.456, 0.406]
+    std = [0.229, 0.224, 0.225]
+    img_mean = np.array(mean).reshape((3, 1, 1))
+    img_std = np.array(std).reshape((3, 1, 1))
+    img *= img_std
+    img += img_mean
+    img *= 255
+    img = np.uint8(img.transpose((0, 2, 3, 1)))
+    return img
+
+
+def extract_img_paths(directory):
+    IMG_EXTENSIONS = ('.jpg', '.jpeg', '.png', '.ppm', '.bmp', '.pgm', '.tif',
+                      '.tiff', '.webp')
+    img_paths = []
+    img_names = []
+    for file in os.listdir(directory):
+        if file.lower().endswith(IMG_EXTENSIONS):
+            img_paths.append(os.path.join(directory, file))
+            img_names.append(file)
+    return img_paths, img_names
+
+
+def preprocess_inputs(inputs, save_path, model_input_shape):
+    if isinstance(inputs, str):
+        imgs = read_image(inputs, crop_size=model_input_shape[1])
+        data = preprocess_image(imgs)
+        if save_path is None or isinstance(save_path, str):
+            save_path = [save_path]
+    elif bool(list) and isinstance(inputs, list) and all(
+            isinstance(elem, str) for elem in inputs):
+        imgs = []
+        for fp in inputs:
+            img = read_image(fp, crop_size=model_input_shape[1])
+            imgs.append(img)
+        imgs = np.concatenate(imgs)
+        data = preprocess_image(imgs)
+        if save_path is None:
+            save_path = [None] * len(imgs)
+    else:
+        if len(inputs.shape) == 3:
+            inputs = np.expand_dims(inputs, axis=0)
+        if np.issubdtype(inputs.dtype, np.integer):
+            imgs = inputs.copy()
+            data = preprocess_image(inputs)
+        else:
+            imgs = restore_image(inputs.copy())
+            data = inputs
+        if save_path is None:
+            save_path = [None] * len(imgs)
+    return imgs, data, save_path
